#!/usr/bin/env python2.7

# This is the main script of confidence score calculation. It needs biogrid publication id and 
# number of interactions per publication as input files. It also needs the threshold to determine
# low/high throughput. 

import sys

# Confidence for low-throughput set. 
def locount_to_score(locount):
    if locount == 1:
        return .8
    elif locount == 2:
        return .9
    elif locount > 2:
        return .95
    else:
        return 0

# Confiden for high-throughput set. 
def hicount_to_score(hicount):
    if hicount == 1:
        return .25
    elif hicount == 2:
        return .5
    elif hicount == 3:
        return .75
    elif hicount > 3:
        return .85
    else:
        return 0

def main(argv):

    if len(argv) < 4:
        help =  "./weight_by_lit [publication file]"
        help += " [count(interactions/pub) file]"
        help += " [confidence file]"
        print help
        exit()

    #pubname = biogrid publication file, it contains three columns, first two columns are protein IDs of 
    # one protein-protein interaction. The third column is the reference publication ID in biogrid.
    #countname = number of protein-protein interactions in each biogrid publication. It contains two columns,
    # first column is publication ID, second column is the number of interactions.
    #confname = output confidence score file name. It contains three columns, firt two columns are protein IDs,
    # third column is confidence score.

    pubname = argv[1]
    countname = argv[2]
    confname = argv[3]

    pubinfile = open(pubname, 'r')
    countinfile = open(countname, 'r')
    confoutfile = open(confname, 'w')

    # thresh = threshold to distinguish high/low throughput
    thresh = eval(raw_input("Threshold: "))

    pubcounts = {}
    hicount = {}
    hicountset = set()
    locount = {}
    locountset = set()
    nameset = set()

    for line in countinfile:
        cols = line.split('\t')
        pub = cols[0]
        count = cols[1][:-1]
        pubcounts[pub] = count

    for line in pubinfile:
        cols = line.split('\t')
        nameA = cols[0]
        nameB = cols[1]
        nameslist = [nameA, nameB]
        nameslist.sort()
        names = (nameslist[0], nameslist[1])
        pub = cols[2][:-1]
        if int(pubcounts[pub]) > thresh:
            if names in hicountset:
                hicount[names] += 1
            else:
                hicountset.add(names)
                hicount[names] = 1
        else:
            if names in locountset:
                locount[names] += 1
            else:
                locountset.add(names)
                locount[names] = 1

        if not names in nameset:
            nameset.add(names)

    # Calculate the confidence score of each protein-protein interaction in low-throughput set and 
    # high-throughput set. Confidence score of interaction in both sets = confidence score of 
    # low-throughput + 0.05 * confidence score of high-throughput. Max confidence score is 0.95.
    for names in nameset:
        if names in hicountset and names not in locountset:
            hiscore = hicount_to_score(int(hicount[names]))
            score = hiscore
        elif names not in hicountset and names in locountset:
            loscore = locount_to_score(int(locount[names]))
            score = loscore
        elif names in hicountset and names in locountset:
            loscore = locount_to_score(int(locount[names]))
            score = loscore + int(hicount[names])*0.05
            if score > 0.95:
                score = 0.95
        else:
            score = 0


        names = '\t'.join(names)
        confoutfile.write(names)
        confoutfile.write('\t')
        confoutfile.write(str(score))
        confoutfile.write('\n')

if __name__ == "__main__":
    main(sys.argv) 
